"""API exceptions."""


class EdgeLabAPIError(Exception):
    """Base exception for EdgeLab API errors."""

    pass


class AuthenticationError(EdgeLabAPIError):
    """Authentication failed."""

    pass


class UnauthorizedError(EdgeLabAPIError):
    """Unauthorized access."""

    pass


class NotFoundError(EdgeLabAPIError):
    """Resource not found."""

    pass


class ValidationError(EdgeLabAPIError):
    """Validation error."""

    pass


class ServerError(EdgeLabAPIError):
    """Server error."""

    pass


class NetworkError(EdgeLabAPIError):
    """Network error."""

    pass



"""Base HTTP client for EdgeLab API."""

import requests
from typing import Optional, Dict, Any
from requests.adapters import HTTPAdapter
from urllib3.util.retry import Retry

from edgelab.config import get_settings
from edgelab.api.exceptions import (
    AuthenticationError,
    UnauthorizedError,
    NotFoundError,
    ValidationError,
    ServerError,
    NetworkError,
)


class EdgeLabClient:
    """Base HTTP client for EdgeLab API."""

    def __init__(self, base_url: Optional[str] = None):
        """Initialize HTTP client.

        Args:
            base_url: API base URL (defaults to settings)
        """
        settings = get_settings()
        self.base_url = base_url or settings.api_base_url
        self.session = self._create_session()

    def _create_session(self) -> requests.Session:
        """Create requests session with retry logic."""
        session = requests.Session()

        # Configure retry strategy
        retry_strategy = Retry(
            total=3,
            backoff_factor=1,
            status_forcelist=[429, 500, 502, 503, 504],
            allowed_methods=["HEAD", "GET", "OPTIONS", "POST"],
        )

        adapter = HTTPAdapter(max_retries=retry_strategy)
        session.mount("http://", adapter)
        session.mount("https://", adapter)

        return session

    def _get_headers(self, authenticated: bool = False) -> Dict[str, str]:
        """Get request headers.

        Args:
            authenticated: Include authentication header

        Returns:
            Headers dict
        """
        headers = {
            "Content-Type": "application/json",
            "Accept": "application/json",
            "User-Agent": "EdgeLab-CLI/0.1.0",
        }

        if authenticated:
            settings = get_settings()
            access_token = settings.get_access_token()
            if access_token:
                headers["Authorization"] = f"Bearer {access_token}"

        return headers

    def _handle_response(self, response: requests.Response) -> Any:
        """Handle API response and raise appropriate exceptions.

        Args:
            response: requests Response object

        Returns:
            Response JSON data

        Raises:
            Various EdgeLabAPIError subclasses
        """
        try:
            # Try to parse JSON response
            data = response.json()
        except ValueError:
            data = {"message": response.text}

        # Handle error status codes
        if response.status_code == 401:
            raise UnauthorizedError(data.get("error", "Unauthorized"))
        elif response.status_code == 403:
            raise AuthenticationError(data.get("error", "Forbidden"))
        elif response.status_code == 404:
            raise NotFoundError(data.get("error", "Not found"))
        elif response.status_code == 400:
            raise ValidationError(data.get("error", "Validation error"))
        elif response.status_code == 409:
            raise ValidationError(data.get("error", "Conflict"))
        elif response.status_code >= 500:
            raise ServerError(data.get("error", "Server error"))
        elif not response.ok:
            raise ServerError(f"HTTP {response.status_code}: {data.get('error', 'Unknown error')}")

        return data

    def get(self, endpoint: str, authenticated: bool = False, **kwargs) -> Any:
        """Make GET request.

        Args:
            endpoint: API endpoint (e.g., "/api/v1/edgelab/strategies")
            authenticated: Require authentication
            **kwargs: Additional requests kwargs

        Returns:
            Response data
        """
        url = f"{self.base_url}{endpoint}"
        headers = self._get_headers(authenticated=authenticated)

        try:
            response = self.session.get(url, headers=headers, **kwargs)
            return self._handle_response(response)
        except requests.exceptions.ConnectionError as e:
            raise NetworkError(f"Connection failed: {e}")
        except requests.exceptions.Timeout as e:
            raise NetworkError(f"Request timeout: {e}")

    def post(
        self, endpoint: str, data: Optional[Dict] = None, authenticated: bool = False, **kwargs
    ) -> Any:
        """Make POST request.

        Args:
            endpoint: API endpoint
            data: Request body (will be JSON encoded)
            authenticated: Require authentication
            **kwargs: Additional requests kwargs

        Returns:
            Response data
        """
        url = f"{self.base_url}{endpoint}"
        headers = self._get_headers(authenticated=authenticated)

        try:
            response = self.session.post(url, json=data, headers=headers, **kwargs)
            return self._handle_response(response)
        except requests.exceptions.ConnectionError as e:
            raise NetworkError(f"Connection failed: {e}")
        except requests.exceptions.Timeout as e:
            raise NetworkError(f"Request timeout: {e}")

    def put(
        self, endpoint: str, data: Optional[Dict] = None, authenticated: bool = False, **kwargs
    ) -> Any:
        """Make PUT request."""
        url = f"{self.base_url}{endpoint}"
        headers = self._get_headers(authenticated=authenticated)

        try:
            response = self.session.put(url, json=data, headers=headers, **kwargs)
            return self._handle_response(response)
        except requests.exceptions.ConnectionError as e:
            raise NetworkError(f"Connection failed: {e}")
        except requests.exceptions.Timeout as e:
            raise NetworkError(f"Request timeout: {e}")

    def delete(self, endpoint: str, authenticated: bool = False, **kwargs) -> Any:
        """Make DELETE request."""
        url = f"{self.base_url}{endpoint}"
        headers = self._get_headers(authenticated=authenticated)

        try:
            response = self.session.delete(url, headers=headers, **kwargs)
            return self._handle_response(response)
        except requests.exceptions.ConnectionError as e:
            raise NetworkError(f"Connection failed: {e}")
        except requests.exceptions.Timeout as e:
            raise NetworkError(f"Request timeout: {e}")



"""Strategy analysis command."""

import sys
import hashlib
import time
from pathlib import Path
from typing import Optional

import click
from rich.progress import Progress, SpinnerColumn, TextColumn, BarColumn, TimeElapsedColumn
from rich.table import Table
from rich import box
from rich.panel import Panel

from edgelab.api.client import EdgeLabClient
from edgelab.api.exceptions import EdgeLabAPIError, UnauthorizedError, NetworkError
from edgelab.config import get_settings
from edgelab.utils import console, print_success, print_error, print_warning, print_info


def cmd_analyze(
    strategy_file: str,
    symbols: str,
    start_date: str,
    end_date: str,
    ml: bool,
    resolution: str,
    asset_type: str,
):
    """Analyze trading strategy on EdgeLab Cloud.

    Args:
        strategy_file: Path to strategy Python file
        symbols: Comma-separated list of symbols (e.g., "SPY,TSLA")
        start_date: Start date (YYYY-MM-DD)
        end_date: End date (YYYY-MM-DD)
        ml: Enable ML optimization
        resolution: Bar resolution (1m, 5m, 15m, 1h, 1d)
        asset_type: Asset type (equity, crypto, forex)
    """
    settings = get_settings()

    # Check authentication
    if not settings.is_authenticated():
        console.print()
        print_error("Not logged in")
        console.print()
        console.print("[dim]Please login first:[/dim]")
        console.print("  [cyan]edgelab auth login[/cyan]")
        console.print()
        sys.exit(1)

    console.print()
    console.print("[bold cyan]📊 EdgeLab Strategy Analysis[/bold cyan]")
    console.print()

    # Read strategy file
    strategy_path = Path(strategy_file)
    if not strategy_path.exists():
        print_error(f"Strategy file not found: {strategy_file}")
        console.print()
        sys.exit(1)

    try:
        strategy_code = strategy_path.read_text()
    except Exception as e:
        print_error(f"Failed to read strategy file: {e}")
        console.print()
        sys.exit(1)

    # Calculate hash
    code_hash = hashlib.sha256(strategy_code.encode()).hexdigest()

    # Extract strategy name and version (simple parsing)
    strategy_name = strategy_path.stem  # Use filename as default
    strategy_version = "v1"  # Default version

    # Parse symbols
    symbol_list = [s.strip().upper() for s in symbols.split(",")]

    # Display analysis details
    info_table = Table(box=box.SIMPLE, show_header=False, padding=(0, 1))
    info_table.add_column("Key", style="bold yellow", width=18)
    info_table.add_column("Value", style="white")

    info_table.add_row("Strategy", strategy_path.name)
    info_table.add_row("Name", strategy_name)
    info_table.add_row("Version", strategy_version)
    info_table.add_row("Code Hash", code_hash[:16] + "...")
    info_table.add_row("", "")
    info_table.add_row("Symbols", ", ".join(symbol_list))
    info_table.add_row("Date Range", f"{start_date} to {end_date}")
    info_table.add_row("Resolution", resolution)
    info_table.add_row("Asset Type", asset_type)
    info_table.add_row("ML Enabled", "Yes" if ml else "No")

    console.print(info_table)
    console.print()

    # Submit to API
    console.print("[dim]Submitting to EdgeLab Cloud...[/dim]")

    try:
        client = EdgeLabClient()
        response = client.post(
            "/api/v1/edgelab/analysis",
            data={
                "strategy_code": strategy_code,
                "strategy_name": strategy_name,
                "strategy_version": strategy_version,
                "code_hash": code_hash,
                "symbols": symbol_list,
                "start_date": start_date,
                "end_date": end_date,
                "asset_type": asset_type,
                "resolution": resolution,
                "ml_enabled": ml,
            },
            authenticated=True,
        )

        # API wraps response in "data" field
        data = response.get("data", response)
        workflow_id = data["workflow_id"]
        status = data["status"]

        console.print()
        print_success("Analysis job submitted!")
        print_info(f"Workflow ID: {workflow_id}")
        print_info(f"Status: {status}")
        console.print()

        # Poll for results
        console.print("[bold yellow]Running analysis on EdgeLab Cloud...[/bold yellow]")
        console.print("[dim]This may take 2-5 minutes depending on data size and ML settings[/dim]")
        console.print()

        poll_workflow(client, workflow_id)

    except UnauthorizedError:
        console.print()
        print_error("Authentication failed")
        console.print()
        console.print("[dim]Your session may have expired. Please login again:[/dim]")
        console.print("  [cyan]edgelab auth login[/cyan]")
        console.print()
        sys.exit(1)

    except NetworkError as e:
        console.print()
        print_error(f"Network error: {e}")
        console.print()
        sys.exit(1)

    except EdgeLabAPIError as e:
        console.print()
        print_error(f"API error: {e}")
        console.print()
        sys.exit(1)


def poll_workflow(client: EdgeLabClient, workflow_id: str):
    """Poll workflow status and display progress.

    Args:
        client: HTTP client
        workflow_id: Workflow ID to poll
    """
    with Progress(
        SpinnerColumn(),
        TextColumn("[progress.description]{task.description}"),
        BarColumn(),
        TextColumn("[progress.percentage]{task.percentage:>3.0f}%"),
        TimeElapsedColumn(),
        console=console,
    ) as progress:
        task = progress.add_task("[cyan]Processing...", total=100)

        while True:
            try:
                response = client.get(
                    f"/api/v1/edgelab/workflows/{workflow_id}",
                    authenticated=True,
                )

                # API wraps response in "data" field
                workflow = response.get("data", response)
                status = workflow["status"]
                current_progress = workflow.get("progress", 0)

                # Update progress bar
                progress.update(task, completed=current_progress)

                if status == "completed":
                    progress.update(task, completed=100)
                    console.print()
                    print_success("Analysis completed!")
                    console.print()

                    # Display results
                    display_results(workflow)
                    break

                elif status == "failed":
                    console.print()
                    print_error("Analysis failed")
                    error_msg = workflow.get("error", "Unknown error")
                    console.print(f"[red]{error_msg}[/red]")
                    console.print()
                    sys.exit(1)

                # Continue polling
                time.sleep(2)

            except KeyboardInterrupt:
                console.print()
                print_warning("Interrupted by user")
                console.print()
                console.print("[dim]Analysis is still running on the server.[/dim]")
                console.print(
                    f"[dim]Check status with:[/dim] [cyan]edgelab results show {workflow_id}[/cyan]"
                )
                console.print()
                sys.exit(0)

            except Exception as e:
                console.print()
                print_error(f"Failed to poll status: {e}")
                console.print()
                sys.exit(1)


def display_results(workflow_data: dict):
    """Display analysis results in rich format.

    Args:
        workflow_data: Workflow data from API
    """
    metadata = workflow_data.get("metadata", {})

    if not metadata:
        print_warning("No results data available")
        return

    # Display backtest results
    if "backtest" in metadata:
        backtest = metadata["backtest"]

        table = Table(
            title="[bold cyan]BACKTEST RESULTS[/bold cyan]",
            box=box.ROUNDED,
            show_header=False,
        )
        table.add_column("Metric", style="bold yellow", width=25)
        table.add_column("Value", style="white")

        table.add_row("Total Trades", str(backtest.get("total_trades", 0)))
        table.add_row("Win Rate", f"{backtest.get('win_rate', 0):.1%}")
        table.add_row("Total P&L", f"${backtest.get('total_pnl', 0):,.2f}")
        table.add_row("Sharpe Ratio", f"{backtest.get('sharpe_ratio', 0):.2f}")
        table.add_row("Max Drawdown", f"{backtest.get('max_drawdown', 0):.1%}")

        console.print(table)
        console.print()

    # Show workflow ID for detailed results
    console.print("[bold yellow]📋 View detailed results:[/bold yellow]")
    console.print(f"  [cyan]edgelab results show {workflow_data['workflow_id']}[/cyan]")
    console.print()



"""
EMA Crossover Strategy

A trend-following strategy using exponential moving average crossovers.

Entry Rules:
- Buy when fast EMA crosses above slow EMA (golden cross)
- Sell when fast EMA crosses below slow EMA (death cross)

Parameters:
- Fast EMA: 12 periods
- Slow EMA: 26 periods

Exit Rules:
- 3% stop loss
- 6% take profit
"""

from typing import Optional


class EMACrossover:
    """Trend following strategy using EMA crossovers."""

    def __init__(self):
        """Initialize strategy state."""
        self.prev_fast_ema = None
        self.prev_slow_ema = None

    @property
    def name(self) -> str:
        """Strategy name."""
        return "ema_crossover"

    @property
    def version(self) -> str:
        """Strategy version."""
        return "v1"

    def on_bar(self, bar) -> Optional[str]:
        """Process each bar and generate trading signals.

        Args:
            bar: OHLCV bar with time, open, high, low, close, volume

        Returns:
            "long" for buy signal (golden cross)
            "short" for sell signal (death cross)
            None for no signal
        """
        # Calculate EMAs
        fast_ema = self.indicators.ema(period=12)
        slow_ema = self.indicators.ema(period=26)

        # Need previous values to detect crossover
        if self.prev_fast_ema is None or self.prev_slow_ema is None:
            self.prev_fast_ema = fast_ema
            self.prev_slow_ema = slow_ema
            return None

        signal = None

        # Golden cross: fast EMA crosses above slow EMA
        if self.prev_fast_ema <= self.prev_slow_ema and fast_ema > slow_ema:
            signal = "long"

        # Death cross: fast EMA crosses below slow EMA
        elif self.prev_fast_ema >= self.prev_slow_ema and fast_ema < slow_ema:
            signal = "short"

        # Update previous values
        self.prev_fast_ema = fast_ema
        self.prev_slow_ema = slow_ema

        return signal

    def stop_loss(self, entry_price: float) -> float:
        """Calculate stop loss price.

        Args:
            entry_price: Price at which position was entered

        Returns:
            Stop loss price (3% below entry)
        """
        return entry_price * 0.97

    def take_profit(self, entry_price: float) -> float:
        """Calculate take profit price.

        Args:
            entry_price: Price at which position was entered

        Returns:
            Take profit price (6% above entry)
        """
        return entry_price * 1.06



"""
Simple RSI Mean Reversion Strategy

A basic momentum strategy using the Relative Strength Index (RSI) to identify
oversold and overbought conditions.

Entry Rules:
- Buy when RSI < 30 (oversold)
- Sell when RSI > 70 (overbought)

Exit Rules:
- 2% stop loss
- 5% take profit
"""

from typing import Optional


class SimpleRSI:
    """Mean reversion strategy using RSI indicator."""

    def __init__(self):
        """Initialize strategy state."""
        pass

    @property
    def name(self) -> str:
        """Strategy name."""
        return "simple_rsi"

    @property
    def version(self) -> str:
        """Strategy version."""
        return "v1"

    def on_bar(self, bar) -> Optional[str]:
        """Process each bar and generate trading signals.

        Args:
            bar: OHLCV bar with time, open, high, low, close, volume

        Returns:
            "long" for buy signal
            "short" for sell signal
            None for no signal
        """
        # Calculate RSI using 14-period lookback
        rsi = self.indicators.rsi(period=14)

        # Trading logic
        if rsi < 30:
            return "long"  # Oversold - buy
        elif rsi > 70:
            return "short"  # Overbought - sell
        else:
            return None  # No signal - hold

    def stop_loss(self, entry_price: float) -> float:
        """Calculate stop loss price.

        Args:
            entry_price: Price at which position was entered

        Returns:
            Stop loss price (2% below entry)
        """
        return entry_price * 0.98

    def take_profit(self, entry_price: float) -> float:
        """Calculate take profit price.

        Args:
            entry_price: Price at which position was entered

        Returns:
            Take profit price (5% above entry)
        """
        return entry_price * 1.05



"""EdgeLab CLI - Main entry point with rich-click interface."""

import sys
from typing import Optional

import click
import rich_click as click
from rich.console import Console
from rich.panel import Panel
from rich.table import Table
from rich import box

from edgelab import __version__

# Configure rich-click for beautiful output
click.rich_click.USE_RICH_MARKUP = True
click.rich_click.USE_MARKDOWN = True
click.rich_click.SHOW_ARGUMENTS = True
click.rich_click.GROUP_ARGUMENTS_OPTIONS = True
click.rich_click.STYLE_ERRORS_SUGGESTION = "yellow italic"
click.rich_click.ERRORS_SUGGESTION = "Try running the '--help' flag for more information."
click.rich_click.ERRORS_EPILOGUE = ""
click.rich_click.MAX_WIDTH = 100
click.rich_click.STYLE_OPTION = "bold cyan"
click.rich_click.STYLE_ARGUMENT = "bold yellow"
click.rich_click.STYLE_COMMAND = "bold green"

# Console instance for rich output
console = Console()


def print_banner():
    """Print EdgeLab banner."""
    banner = """
[bold cyan]╔═══════════════════════════════════════════════════════╗[/bold cyan]
[bold cyan]║[/bold cyan]                  [bold white]EdgeLab CLI[/bold white]                     [bold cyan]║[/bold cyan]
[bold cyan]║[/bold cyan]     [dim]Algorithmic Trading Strategy Analysis[/dim]       [bold cyan]║[/bold cyan]
[bold cyan]╚═══════════════════════════════════════════════════════╝[/bold cyan]
    """
    console.print(banner)


@click.group(
    invoke_without_command=True,
    help="""
    [bold cyan]EdgeLab CLI[/bold cyan] - Algorithmic Trading Strategy Analysis Platform

    Write Python strategies locally, analyze them on EdgeLab Cloud, and view
    results with beautiful formatted output.

    [bold yellow]Quick Start:[/bold yellow]

      $ edgelab signup          # Create account
      $ edgelab login           # Login
      $ edgelab init my-strategies   # Create workspace
      $ edgelab analyze strategies/simple_rsi.py SPY 2023-01-01 2023-12-31

    [bold yellow]Example Commands:[/bold yellow]

      $ edgelab analyze strategies/my_rsi.py SPY,TSLA 2023-01-01 2023-12-31
      $ edgelab analyze --ml strategies/ml_rsi.py SPY 2023-01-01 2023-12-31
      $ edgelab results list
      $ edgelab strategies list
    """,
)
@click.version_option(
    version=__version__,
    prog_name="edgelab",
    message="%(prog)s version %(version)s",
)
@click.pass_context
def main(ctx: click.Context):
    """EdgeLab CLI - Algorithmic Trading Strategy Analysis."""
    # If no command provided, show banner and help
    if ctx.invoked_subcommand is None:
        print_banner()
        console.print()
        click.echo(ctx.get_help())


@main.command(
    help="""
    [bold]Display EdgeLab CLI version and system information.[/bold]

    Shows the current version of EdgeLab CLI installed on your system,
    along with Python version and platform information.
    """
)
def version():
    """Display version information."""
    import platform

    # Create info table
    table = Table(
        title="[bold cyan]EdgeLab CLI Version Info[/bold cyan]",
        box=box.ROUNDED,
        show_header=False,
        padding=(0, 1),
    )

    table.add_column("Key", style="bold yellow", width=20)
    table.add_column("Value", style="white")

    table.add_row("EdgeLab CLI", f"v{__version__}")
    table.add_row("Python", platform.python_version())
    table.add_row("Platform", platform.platform())
    table.add_row("Architecture", platform.machine())

    console.print()
    console.print(table)
    console.print()

    # Print status
    console.print("✨ EdgeLab CLI is installed and ready!", style="bold green")
    console.print()
    console.print("Next steps:", style="bold yellow")
    console.print("  1. Sign up:  [cyan]edgelab signup[/cyan]")
    console.print("  2. Login:    [cyan]edgelab login[/cyan]")
    console.print("  3. Get help: [cyan]edgelab --help[/cyan]")
    console.print()


# Placeholder commands (will implement next)
@main.group(
    help="""
    [bold]Authentication commands.[/bold]

    Manage your EdgeLab account authentication.
    """
)
def auth():
    """Authentication commands."""
    pass


@auth.command()
def signup():
    """Create a new EdgeLab account."""
    from edgelab.commands.auth import cmd_signup

    cmd_signup()


@auth.command()
def login():
    """Login to EdgeLab."""
    from edgelab.commands.auth import cmd_login

    cmd_login()


@auth.command()
def logout():
    """Logout from EdgeLab."""
    from edgelab.commands.auth import cmd_logout

    cmd_logout()


@auth.command()
def whoami():
    """Show current authentication status."""
    from edgelab.commands.auth import cmd_whoami

    cmd_whoami()


@main.command(
    help="""
    [bold]Initialize a new EdgeLab workspace.[/bold]

    Creates a new workspace with example strategies to get you started.
    """
)
@click.argument("folder", type=str)
def init(folder: str):
    """Initialize workspace with example strategies."""
    from edgelab.commands.workspace import cmd_init

    cmd_init(folder)


@main.command(
    help="""
    [bold]Analyze a trading strategy on EdgeLab Cloud.[/bold]

    Submits your strategy for server-side analysis with 4 engines:
    - Backtest (in-sample)
    - Walk-Forward (out-of-sample validation)
    - Monte Carlo (1000 simulations)
    - Stress Testing (historical crises)
    """
)
@click.argument("strategy-file", type=click.Path(exists=True))
@click.argument("symbols", type=str)
@click.argument("start-date", type=str)
@click.argument("end-date", type=str)
@click.option(
    "--ml",
    is_flag=True,
    help="Enable ML optimization (trains model to find optimal parameters)",
)
@click.option(
    "--resolution",
    type=click.Choice(["1m", "5m", "15m", "1h", "1d"]),
    default="5m",
    help="Bar resolution (default: 5m)",
)
@click.option(
    "--asset-type",
    type=click.Choice(["equity", "crypto", "forex"]),
    default="equity",
    help="Asset type (default: equity)",
)
def analyze(
    strategy_file: str,
    symbols: str,
    start_date: str,
    end_date: str,
    ml: bool,
    resolution: str,
    asset_type: str,
):
    """Analyze trading strategy."""
    from edgelab.commands.analyze import cmd_analyze

    cmd_analyze(strategy_file, symbols, start_date, end_date, ml, resolution, asset_type)


@main.group(
    help="""
    [bold]Manage analysis results.[/bold]

    View and explore your backtesting results.
    """
)
def results():
    """Results management commands."""
    pass


@results.command(name="list")
def results_list():
    """List all analysis runs."""
    console.print("[yellow]⚠️  Coming soon![/yellow] List results.")


@results.command()
@click.argument("workflow-id", type=str)
@click.option(
    "--detail",
    is_flag=True,
    help="Show detailed trade-by-trade breakdown",
)
def show(workflow_id: str, detail: bool):
    """Show detailed results for a workflow."""
    from edgelab.commands.results import cmd_results_show

    cmd_results_show(workflow_id, show_detail=detail)


@main.group(
    help="""
    [bold]Manage trading strategies.[/bold]

    View and manage your strategies stored on EdgeLab Cloud.
    """
)
def strategies():
    """Strategy management commands."""
    pass


@strategies.command(name="list")
def strategies_list():
    """List all your strategies."""
    console.print("[yellow]⚠️  Coming soon![/yellow] List strategies.")


@strategies.command()
@click.argument("name", type=str)
@click.argument("version", type=str)
def show(name: str, version: str):
    """Show strategy details."""
    console.print(f"[yellow]⚠️  Coming soon![/yellow] Show strategy: {name} {version}")


if __name__ == "__main__":
    main()



"""Utility functions and helpers."""

from edgelab.utils.console import console, print_success, print_error, print_warning, print_info

__all__ = ["console", "print_success", "print_error", "print_warning", "print_info"]



"""Workspace initialization command."""

import sys
import shutil
from pathlib import Path

from edgelab.utils import console, print_success, print_error, print_info


def cmd_init(folder: str):
    """Initialize EdgeLab workspace with example strategies.

    Args:
        folder: Workspace folder name
    """
    console.print()
    console.print(f"[bold cyan]Creating EdgeLab Workspace:[/bold cyan] {folder}")
    console.print()

    workspace_path = Path(folder).resolve()

    # Check if folder already exists
    if workspace_path.exists():
        print_error(f"Folder already exists: {workspace_path}")
        console.print()
        console.print("[dim]Choose a different name or remove the existing folder.[/dim]")
        console.print()
        sys.exit(1)

    try:
        # Create workspace structure
        workspace_path.mkdir(parents=True)
        (workspace_path / "strategies").mkdir()
        (workspace_path / "results").mkdir()

        print_success(f"Created workspace: {workspace_path}")
        console.print()

        # Copy example strategies
        import edgelab.examples
        examples_dir = Path(edgelab.examples.__file__).parent

        example_files = [
            "simple_rsi.py",
            "ema_crossover.py",
        ]

        console.print("[bold yellow]📝 Creating example strategies:[/bold yellow]")
        for filename in example_files:
            src = examples_dir / filename
            dst = workspace_path / "strategies" / filename

            if src.exists():
                shutil.copy(src, dst)
                print_success(f"  strategies/{filename}")
            else:
                print_error(f"  Example not found: {filename}")

        console.print()

        # Create README
        readme_content = """# EdgeLab Workspace

This workspace contains your trading strategies and analysis results.

## Folder Structure

- `strategies/` - Your trading strategy files
- `results/` - Analysis results (future feature)

## Example Strategies

### simple_rsi.py
Mean reversion strategy using RSI indicator.
- Buy when RSI < 30 (oversold)
- Sell when RSI > 70 (overbought)

### ema_crossover.py
Trend following strategy using EMA crossovers.
- Buy on golden cross (fast EMA crosses above slow EMA)
- Sell on death cross (fast EMA crosses below slow EMA)

## Running Analysis

```bash
# Analyze strategy on single symbol
edgelab analyze strategies/simple_rsi.py SPY 2023-01-01 2023-12-31

# Analyze on multiple symbols
edgelab analyze strategies/simple_rsi.py SPY,TSLA,NVDA 2023-01-01 2023-12-31

# With ML optimization
edgelab analyze --ml strategies/simple_rsi.py SPY,QQQ 2023-01-01 2023-12-31
```

## Next Steps

1. Review example strategies
2. Modify parameters or create your own strategy
3. Run analysis with `edgelab analyze`
4. View results with `edgelab results list`

## Documentation

- Full docs: https://docs.edgelab.com
- API reference: https://docs.edgelab.com/api
- Strategy examples: https://docs.edgelab.com/strategies
"""

        readme_path = workspace_path / "README.md"
        readme_path.write_text(readme_content)
        print_success("  README.md")

        console.print()
        console.print("[bold green]✨ Workspace ready![/bold green]")
        console.print()

        # Show next steps
        console.print("[bold yellow]Next steps:[/bold yellow]")
        console.print(f"  1. Navigate to workspace:")
        console.print(f"     [cyan]cd {folder}[/cyan]")
        console.print()
        console.print(f"  2. Review example strategies:")
        console.print(f"     [cyan]cat strategies/simple_rsi.py[/cyan]")
        console.print()
        console.print(f"  3. Run your first analysis:")
        console.print(f"     [cyan]edgelab analyze strategies/simple_rsi.py SPY 2023-01-01 2023-12-31[/cyan]")
        console.print()

    except Exception as e:
        console.print()
        print_error(f"Failed to create workspace: {e}")
        console.print()
        sys.exit(1)



"""Results display commands."""

import json
import sys
from datetime import datetime
from typing import Dict, Any, Optional

from edgelab.api.client import EdgeLabClient
from edgelab.api.exceptions import EdgeLabAPIError, UnauthorizedError, NotFoundError
from edgelab.config import get_settings
from edgelab.utils import console, print_success, print_error, print_warning, print_info
from rich.table import Table
from rich.panel import Panel
from rich import box


def cmd_results_show(workflow_id: str, show_detail: bool = False):
    """Show detailed results for a workflow.

    Args:
        workflow_id: Workflow UUID
        show_detail: If True, show detailed trade-by-trade breakdown
    """
    settings = get_settings()

    # Check authentication
    if not settings.is_authenticated():
        console.print()
        print_error("Not logged in")
        console.print()
        console.print("[dim]Please login first:[/dim]")
        console.print("  [cyan]edgelab auth login[/cyan]")
        console.print()
        sys.exit(1)

    try:
        # Fetch workflow from API
        client = EdgeLabClient()
        response = client.get(
            f"/api/v1/edgelab/workflows/{workflow_id}",
            authenticated=True,
        )

        # API wraps response in "data" field
        workflow = response.get("data", response)

        # Check status
        status = workflow.get("status")
        if status == "pending" or status == "processing":
            console.print()
            print_warning(f"Workflow is still {status}")
            console.print()
            console.print(f"[dim]Progress: {workflow.get('progress', 0)}%[/dim]")
            console.print()
            sys.exit(0)

        if status == "failed":
            console.print()
            print_error("Analysis failed")
            error_msg = workflow.get("error", "Unknown error")
            console.print(f"[red]{error_msg}[/red]")
            console.print()
            sys.exit(1)

        if status != "completed":
            console.print()
            print_warning(f"Unexpected status: {status}")
            console.print()
            sys.exit(1)

        # Display results
        display_results(workflow, show_detail=show_detail)

    except NotFoundError:
        console.print()
        print_error(f"Workflow not found: {workflow_id}")
        console.print()
        sys.exit(1)
    except UnauthorizedError:
        console.print()
        print_error("Unauthorized - please login again")
        console.print()
        console.print("[dim]Login:[/dim] [cyan]edgelab auth login[/cyan]")
        console.print()
        sys.exit(1)
    except EdgeLabAPIError as e:
        console.print()
        print_error(f"API error: {e}")
        console.print()
        sys.exit(1)
    except Exception as e:
        console.print()
        print_error(f"Failed to fetch results: {e}")
        console.print()
        sys.exit(1)


def display_results(workflow_data: Dict[str, Any], show_detail: bool = False):
    """Display analysis results in rich format.

    Args:
        workflow_data: Workflow data from API
        show_detail: If True, show detailed trade-by-trade breakdown
    """
    metadata = workflow_data.get("metadata")
    if not metadata:
        print_warning("No results data available")
        return

    # Parse metadata JSON if it's a string
    if isinstance(metadata, str):
        try:
            metadata = json.loads(metadata)
        except json.JSONDecodeError:
            print_error("Invalid metadata format")
            return

    # Extract results
    results = metadata.get("results", {})
    if not results:
        print_warning("No results found in metadata")
        return

    console.print()
    console.print(Panel.fit("[bold cyan]📊 Analysis Results[/bold cyan]", border_style="cyan"))
    console.print()

    # Display results for each symbol
    for symbol, symbol_results in results.items():
        display_symbol_results(symbol, symbol_results, show_detail=show_detail)

    # Display workflow info
    console.print()
    console.print(f"[dim]Workflow ID:[/dim] {workflow_data.get('workflow_id')}")
    console.print(f"[dim]Status:[/dim] {workflow_data.get('status')}")
    console.print()


def display_symbol_results(symbol: str, symbol_results: Dict[str, Any], show_detail: bool = False):
    """Display results for a single symbol.

    Args:
        symbol: Symbol name
        symbol_results: Results dict for this symbol
        show_detail: If True, show detailed trade-by-trade breakdown
    """
    console.print(f"[bold yellow]📈 Symbol: {symbol}[/bold yellow]")
    console.print()

    # Backtest results
    if "backtest" in symbol_results:
        display_backtest_results(symbol_results["backtest"], show_detail=show_detail)

    # Walkforward results
    if "walkforward" in symbol_results:
        display_walkforward_results(symbol_results["walkforward"], show_detail=show_detail)

    # Monte Carlo results
    if "montecarlo" in symbol_results:
        display_montecarlo_results(symbol_results["montecarlo"])

    # Stress test results
    if "stress" in symbol_results:
        display_stress_results(symbol_results["stress"], show_detail=show_detail)

    console.print()


def display_backtest_results(backtest: Dict[str, Any], show_detail: bool = False):
    """Display backtest results.

    Args:
        backtest: Backtest results dict
        show_detail: If True, show detailed trade-by-trade breakdown
    """
    metrics = backtest.get("metrics", {})
    trades = backtest.get("trades", [])

    table = Table(
        title="[bold cyan]Backtest[/bold cyan]",
        box=box.ROUNDED,
        show_header=False,
        padding=(0, 2),
    )
    table.add_column("Metric", style="bold yellow", width=25)
    table.add_column("Value", style="white")

    # Core metrics
    if "total_trades" in metrics:
        table.add_row("Total Trades", str(metrics["total_trades"]))
    if "win_rate" in metrics:
        table.add_row("Win Rate", f"{metrics['win_rate']:.1%}")
    if "total_pnl" in metrics:
        pnl = metrics["total_pnl"]
        color = "green" if pnl >= 0 else "red"
        table.add_row("Total P&L", f"[{color}]{pnl:,.2f}[/{color}]")
    if "sharpe_ratio" in metrics:
        table.add_row("Sharpe Ratio", f"{metrics['sharpe_ratio']:.2f}")
    if "max_drawdown" in metrics:
        table.add_row("Max Drawdown", f"{metrics['max_drawdown']:.1%}")
    if "profit_factor" in metrics:
        table.add_row("Profit Factor", f"{metrics['profit_factor']:.2f}")
    if "avg_win" in metrics:
        table.add_row("Avg Win", f"{metrics['avg_win']:,.2f}")
    if "avg_loss" in metrics:
        table.add_row("Avg Loss", f"{metrics['avg_loss']:,.2f}")
    if "max_consecutive_wins" in metrics:
        table.add_row("Max Consecutive Wins", str(metrics["max_consecutive_wins"]))
    if "max_consecutive_losses" in metrics:
        table.add_row("Max Consecutive Losses", str(metrics["max_consecutive_losses"]))

    console.print(table)

    # Display trades table only if --detail flag is set
    if show_detail and trades:
        display_trades_table(trades, title="Backtest Trades")
    elif not show_detail:
        console.print(f"[dim]Total trades: {metrics.get('total_trades', 0)}[/dim]")
    console.print()


def display_trades_table(trades: list, title: str = "Trades"):
    """Display trades in a formatted table.

    Args:
        trades: List of trade dictionaries
        title: Table title
    """
    if not trades:
        return

    table = Table(
        title=f"[bold cyan]{title} ({len(trades)})[/bold cyan]",
        box=box.ROUNDED,
        show_header=True,
        header_style="bold yellow",
        padding=(0, 1),
    )

    # Add columns
    table.add_column("#", style="dim", width=4, justify="right")
    table.add_column("Entry Time", style="cyan", width=16)
    table.add_column("Exit Time", style="cyan", width=16)
    table.add_column("Entry", style="white", width=10, justify="right")
    table.add_column("Exit", style="white", width=10, justify="right")
    table.add_column("Qty", style="dim", width=8, justify="right")
    table.add_column("P&L", style="white", width=12, justify="right")
    table.add_column("Return %", style="white", width=10, justify="right")
    table.add_column("Reason", style="dim", width=12)
    table.add_column("MAE", style="dim", width=8, justify="right")
    table.add_column("MFE", style="dim", width=8, justify="right")

    # Add trade rows
    for idx, trade in enumerate(trades, 1):
        # Parse timestamps
        entry_time = trade.get("entry_time", "")
        exit_time = trade.get("exit_time", "")

        # Format timestamps (remove timezone for display)
        try:
            if entry_time:
                dt = datetime.fromisoformat(entry_time.replace("Z", "+00:00"))
                entry_time = dt.strftime("%Y-%m-%d %H:%M")
            if exit_time:
                dt = datetime.fromisoformat(exit_time.replace("Z", "+00:00"))
                exit_time = dt.strftime("%Y-%m-%d %H:%M")
        except (ValueError, AttributeError):
            pass

        entry_price = trade.get("entry_price", 0)
        exit_price = trade.get("exit_price", 0)
        quantity = trade.get("quantity", 0)
        pnl = trade.get("pnl", 0)
        return_pct = trade.get("return_pct", 0)
        exit_reason = trade.get("exit_reason", "")
        mae = trade.get("mae", 0)
        mfe = trade.get("mfe", 0)

        # Color code P&L
        pnl_color = "green" if pnl >= 0 else "red"
        return_color = "green" if return_pct >= 0 else "red"

        # Format values
        entry_str = f"{entry_price:.2f}" if entry_price else "-"
        exit_str = f"{exit_price:.2f}" if exit_price else "-"
        qty_str = f"{quantity:.2f}" if quantity else "-"
        pnl_str = f"[{pnl_color}]{pnl:,.2f}[/{pnl_color}]"
        return_str = f"[{return_color}]{return_pct:.2f}%[/{return_color}]"
        mae_str = f"{mae:.2f}" if mae else "-"
        mfe_str = f"{mfe:.2f}" if mfe else "-"

        table.add_row(
            str(idx),
            entry_time or "-",
            exit_time or "-",
            entry_str,
            exit_str,
            qty_str,
            pnl_str,
            return_str,
            exit_reason or "-",
            mae_str,
            mfe_str,
        )

    console.print()
    console.print(table)
    console.print()


def display_walkforward_results(walkforward: Dict[str, Any], show_detail: bool = False):
    """Display walkforward results.

    Args:
        walkforward: Walkforward results dict
        show_detail: If True, show detailed trade-by-trade breakdown
    """
    metrics = walkforward.get("combined_metrics", {})
    num_windows = walkforward.get("num_windows", 0)
    trades = walkforward.get("trades", [])

    table = Table(
        title="[bold cyan]Walk-Forward Analysis[/bold cyan]",
        box=box.ROUNDED,
        show_header=False,
        padding=(0, 2),
    )
    table.add_column("Metric", style="bold yellow", width=25)
    table.add_column("Value", style="white")

    if num_windows > 0:
        table.add_row("Windows", str(num_windows))

    # Combined metrics (same structure as backtest)
    if "total_trades" in metrics:
        table.add_row("Total Trades", str(metrics["total_trades"]))
    if "win_rate" in metrics:
        table.add_row("Win Rate", f"{metrics['win_rate']:.1%}")
    if "total_pnl" in metrics:
        pnl = metrics["total_pnl"]
        color = "green" if pnl >= 0 else "red"
        table.add_row("Total P&L", f"[{color}]{pnl:,.2f}[/{color}]")
    if "sharpe_ratio" in metrics:
        table.add_row("Sharpe Ratio", f"{metrics['sharpe_ratio']:.2f}")
    if "max_drawdown" in metrics:
        table.add_row("Max Drawdown", f"{metrics['max_drawdown']:.1%}")

    console.print(table)

    # Display trades table only if --detail flag is set
    if show_detail and trades:
        display_trades_table(trades, title="Walk-Forward Trades")
    elif not show_detail and trades:
        console.print(f"[dim]Total trades: {metrics.get('total_trades', len(trades))}[/dim]")
    console.print()


def display_montecarlo_results(montecarlo: Dict[str, Any]):
    """Display Monte Carlo results.

    Args:
        montecarlo: Monte Carlo results dict
    """
    table = Table(
        title="[bold cyan]Monte Carlo Simulation[/bold cyan]",
        box=box.ROUNDED,
        show_header=False,
        padding=(0, 2),
    )
    table.add_column("Metric", style="bold yellow", width=25)
    table.add_column("Value", style="white")

    if "num_simulations" in montecarlo:
        table.add_row("Simulations", str(montecarlo["num_simulations"]))
    if "pnl_mean" in montecarlo:
        pnl_mean = montecarlo["pnl_mean"]
        color = "green" if pnl_mean >= 0 else "red"
        table.add_row("Mean P&L", f"[{color}]{pnl_mean:,.2f}[/{color}]")
    if "pnl_std" in montecarlo:
        table.add_row("Std Dev P&L", f"{montecarlo['pnl_std']:,.2f}")
    if "sharpe_mean" in montecarlo:
        table.add_row("Mean Sharpe", f"{montecarlo['sharpe_mean']:.2f}")

    console.print(table)
    console.print()


def display_stress_results(stress: Dict[str, Any], show_detail: bool = False):
    """Display stress test results.

    Args:
        stress: Stress test results dict
        show_detail: If True, show detailed trade-by-trade breakdown
    """
    metrics = stress.get("baseline_metrics", {})
    num_scenarios = stress.get("num_scenarios", 0)
    trades = stress.get("trades", [])

    table = Table(
        title="[bold cyan]Stress Test[/bold cyan]",
        box=box.ROUNDED,
        show_header=False,
        padding=(0, 2),
    )
    table.add_column("Metric", style="bold yellow", width=25)
    table.add_column("Value", style="white")

    if num_scenarios > 0:
        table.add_row("Scenarios", str(num_scenarios))

    # Baseline metrics (same structure as backtest)
    if "total_trades" in metrics:
        table.add_row("Total Trades", str(metrics["total_trades"]))
    if "win_rate" in metrics:
        table.add_row("Win Rate", f"{metrics['win_rate']:.1%}")
    if "total_pnl" in metrics:
        pnl = metrics["total_pnl"]
        color = "green" if pnl >= 0 else "red"
        table.add_row("Total P&L", f"[{color}]{pnl:,.2f}[/{color}]")
    if "sharpe_ratio" in metrics:
        table.add_row("Sharpe Ratio", f"{metrics['sharpe_ratio']:.2f}")
    if "max_drawdown" in metrics:
        table.add_row("Max Drawdown", f"{metrics['max_drawdown']:.1%}")

    console.print(table)

    # Display trades table only if --detail flag is set
    if show_detail and trades:
        display_trades_table(trades, title="Stress Test Trades")
    elif not show_detail and trades:
        console.print(f"[dim]Total trades: {metrics.get('total_trades', len(trades))}[/dim]")
    console.print()



"""Authentication commands implementation."""

import sys
import click
from rich.table import Table
from rich import box

from edgelab.api.client import EdgeLabClient
from edgelab.api.auth import AuthAPI
from edgelab.api.exceptions import (
    ValidationError,
    AuthenticationError,
    NetworkError,
    EdgeLabAPIError,
)
from edgelab.config import get_settings
from edgelab.utils import console, print_success, print_error, print_warning, print_info


def cmd_signup():
    """Sign up for new EdgeLab account."""
    console.print()
    console.print("[bold cyan]EdgeLab Sign Up[/bold cyan]")
    console.print()

    # Get email
    email = click.prompt("Email", type=str)

    # Get password (with confirmation)
    password = click.prompt("Password", hide_input=True, confirmation_prompt=True)

    console.print()
    console.print("Creating account...", style="dim")

    try:
        # Call API
        client = EdgeLabClient()
        auth_api = AuthAPI(client)
        result = auth_api.signup(email, password)

        # Save tokens
        settings = get_settings()
        settings.save_auth(
            access_token=result["access_token"],
            refresh_token=result["refresh_token"],
            expires_at=result["expires_at"],
            user={
                "id": result.get("user_id", "unknown"),
                "email": email,
                "role": "free",
            },
        )

        console.print()
        print_success("Account created successfully!")
        print_success(f"Logged in as: {email}")
        print_success(f"Tokens saved to: {settings.auth_file}")
        console.print()

        # Show next steps
        console.print("[bold yellow]Next steps:[/bold yellow]")
        console.print("  1. Initialize workspace:  [cyan]edgelab init my-strategies[/cyan]")
        console.print("  2. Analyze a strategy:    [cyan]edgelab analyze --help[/cyan]")
        console.print()

    except ValidationError as e:
        console.print()
        print_error(f"Signup failed: {e}")
        console.print()
        console.print("[dim]Common issues:[/dim]")
        console.print("  • Email already registered")
        console.print("  • Password too weak (min 8 characters)")
        console.print("  • Invalid email format")
        console.print()
        sys.exit(1)

    except NetworkError as e:
        console.print()
        print_error(f"Network error: {e}")
        console.print()
        console.print("[dim]Please check:[/dim]")
        console.print("  • Internet connection")
        console.print("  • EdgeLab server status")
        console.print()
        sys.exit(1)

    except EdgeLabAPIError as e:
        console.print()
        print_error(f"Signup failed: {e}")
        console.print()
        sys.exit(1)


def cmd_login():
    """Login to EdgeLab."""
    console.print()
    console.print("[bold cyan]EdgeLab Login[/bold cyan]")
    console.print()

    # Get credentials
    email = click.prompt("Email", type=str)
    password = click.prompt("Password", hide_input=True)

    console.print()
    console.print("Authenticating...", style="dim")

    try:
        # Call API
        client = EdgeLabClient()
        auth_api = AuthAPI(client)
        result = auth_api.login(email, password)

        # Save tokens
        settings = get_settings()
        settings.save_auth(
            access_token=result["access_token"],
            refresh_token=result["refresh_token"],
            expires_at=result["expires_at"],
            user={
                "id": result.get("user_id", "unknown"),
                "email": email,
                "role": "free",
            },
        )

        console.print()
        print_success("Login successful!")
        print_success(f"Logged in as: {email}")
        print_success(f"Tokens saved to: {settings.auth_file}")
        console.print()

    except AuthenticationError as e:
        console.print()
        print_error(f"Login failed: Invalid email or password")
        console.print()
        console.print("[dim]Forgot password?[/dim]")
        console.print("  Visit: https://pantry-chef.com/forgot-password")
        console.print()
        sys.exit(1)

    except NetworkError as e:
        console.print()
        print_error(f"Network error: {e}")
        console.print()
        sys.exit(1)

    except EdgeLabAPIError as e:
        console.print()
        print_error(f"Login failed: {e}")
        console.print()
        sys.exit(1)


def cmd_logout():
    """Logout from EdgeLab."""
    settings = get_settings()

    if not settings.is_authenticated():
        console.print()
        print_warning("Not logged in")
        console.print()
        sys.exit(1)

    console.print()
    console.print("Logging out...", style="dim")

    try:
        # Call logout API to invalidate refresh token
        refresh_token = settings.get_refresh_token()
        if refresh_token:
            client = EdgeLabClient()
            auth_api = AuthAPI(client)
            auth_api.logout(refresh_token)

        # Clear local tokens
        settings.clear_auth()

        console.print()
        print_success("Logged out successfully")
        print_info(f"Removed: {settings.auth_file}")
        console.print()

    except Exception as e:
        # Even if API call fails, clear local tokens
        settings.clear_auth()
        console.print()
        print_success("Logged out (local tokens cleared)")
        console.print()


def cmd_whoami():
    """Show current authentication status."""
    settings = get_settings()

    console.print()

    if not settings.is_authenticated():
        print_warning("Not logged in")
        console.print()
        console.print("[dim]To login:[/dim]")
        console.print("  [cyan]edgelab login[/cyan]")
        console.print()
        console.print("[dim]To sign up:[/dim]")
        console.print("  [cyan]edgelab signup[/cyan]")
        console.print()
        sys.exit(1)

    # Get user info
    user = settings.get_user()
    auth = settings.load_auth()

    # Create table
    table = Table(
        title="[bold cyan]EdgeLab Authentication Status[/bold cyan]",
        box=box.ROUNDED,
        show_header=False,
        padding=(0, 1),
    )

    table.add_column("Key", style="bold yellow", width=20)
    table.add_column("Value", style="white")

    table.add_row("Status", "[green]✓ Authenticated[/green]")
    table.add_row("Email", user.get("email", "Unknown"))
    table.add_row("User ID", user.get("id", "Unknown"))
    table.add_row("Plan", user.get("role", "free").capitalize())
    table.add_row("Config", str(settings.auth_file))

    console.print(table)
    console.print()



"""Rich console utilities for beautiful terminal output."""

from rich.console import Console
from rich.panel import Panel
from rich.table import Table
from rich import box

# Global console instance
console = Console()


def print_success(message: str):
    """Print success message with checkmark."""
    console.print(f"✓ {message}", style="bold green")


def print_error(message: str):
    """Print error message with X."""
    console.print(f"✗ {message}", style="bold red")


def print_warning(message: str):
    """Print warning message."""
    console.print(f"⚠️  {message}", style="bold yellow")


def print_info(message: str):
    """Print info message."""
    console.print(f"ℹ️  {message}", style="bold cyan")



"""Authentication API client."""

from typing import Dict, Any
from edgelab.api.client import EdgeLabClient


class AuthAPI:
    """Authentication API endpoints."""

    def __init__(self, client: EdgeLabClient):
        """Initialize auth API.

        Args:
            client: Base HTTP client
        """
        self.client = client

    def signup(self, email: str, password: str) -> Dict[str, Any]:
        """Sign up for new EdgeLab account.

        Args:
            email: User email
            password: User password

        Returns:
            Response data with tokens and user info

        Raises:
            ValidationError: Email already exists or invalid input
            NetworkError: Connection failed
        """
        data = {"email": email, "password": password}
        response = self.client.post("/api/v1/edgelab/auth/signup", data=data)
        return response["data"]  # API wraps response in {"data": {...}}

    def login(self, email: str, password: str) -> Dict[str, Any]:
        """Login to EdgeLab.

        Args:
            email: User email
            password: User password

        Returns:
            Response data with tokens and user info

        Raises:
            AuthenticationError: Invalid credentials
            NetworkError: Connection failed
        """
        data = {"email": email, "password": password}
        response = self.client.post("/api/v1/edgelab/auth/login", data=data)
        return response["data"]

    def refresh_token(self, refresh_token: str) -> Dict[str, Any]:
        """Refresh access token.

        Args:
            refresh_token: Refresh token

        Returns:
            New access and refresh tokens

        Raises:
            UnauthorizedError: Invalid refresh token
        """
        data = {"refresh_token": refresh_token}
        response = self.client.post("/api/v1/edgelab/auth/refresh", data=data)
        return response["data"]

    def logout(self, refresh_token: str) -> Dict[str, Any]:
        """Logout from EdgeLab.

        Args:
            refresh_token: Refresh token to invalidate

        Returns:
            Success message
        """
        data = {"refresh_token": refresh_token}
        return self.client.post("/api/v1/edgelab/auth/logout", data=data)



"""Settings and configuration management for EdgeLab CLI."""

import json
import os
from pathlib import Path
from typing import Optional
from datetime import datetime

from pydantic import BaseModel, Field
from pydantic_settings import BaseSettings


class AuthTokens(BaseModel):
    """Authentication tokens."""

    access_token: str
    refresh_token: str
    expires_at: datetime


class UserInfo(BaseModel):
    """User information."""

    id: str
    email: str
    role: str = "free"


class Settings(BaseSettings):
    """EdgeLab CLI settings."""

    # API Configuration
    api_base_url: str = Field(
        default="https://pantry-chef.com",
        description="EdgeLab API base URL",
    )

    # Local directories
    edgelab_dir: Path = Field(
        default_factory=lambda: Path.home() / ".edgelab",
        description="EdgeLab config directory",
    )

    @property
    def auth_file(self) -> Path:
        """Path to auth tokens file."""
        return self.edgelab_dir / "auth.json"

    @property
    def config_file(self) -> Path:
        """Path to config file."""
        return self.edgelab_dir / "config.json"

    def ensure_dir_exists(self):
        """Ensure EdgeLab directory exists with correct permissions."""
        self.edgelab_dir.mkdir(parents=True, exist_ok=True)
        # Set directory permissions to 700 (rwx------)
        os.chmod(self.edgelab_dir, 0o700)

    def load_auth(self) -> Optional[dict]:
        """Load authentication tokens from file.

        Returns:
            Dict with access_token, refresh_token, expires_at, user
            None if not authenticated
        """
        if not self.auth_file.exists():
            return None

        try:
            with open(self.auth_file, "r") as f:
                return json.load(f)
        except (json.JSONDecodeError, IOError):
            return None

    def save_auth(self, access_token: str, refresh_token: str, expires_at: str, user: dict):
        """Save authentication tokens to file.

        Args:
            access_token: JWT access token
            refresh_token: JWT refresh token
            expires_at: ISO format expiration timestamp
            user: User info dict (id, email, role)
        """
        self.ensure_dir_exists()

        auth_data = {
            "access_token": access_token,
            "refresh_token": refresh_token,
            "expires_at": expires_at,
            "user": user,
        }

        with open(self.auth_file, "w") as f:
            json.dump(auth_data, f, indent=2)

        # Set file permissions to 600 (rw-------)
        os.chmod(self.auth_file, 0o600)

    def clear_auth(self):
        """Clear authentication tokens."""
        if self.auth_file.exists():
            self.auth_file.unlink()

    def is_authenticated(self) -> bool:
        """Check if user is authenticated."""
        auth = self.load_auth()
        return auth is not None and "access_token" in auth

    def get_access_token(self) -> Optional[str]:
        """Get current access token."""
        auth = self.load_auth()
        return auth.get("access_token") if auth else None

    def get_refresh_token(self) -> Optional[str]:
        """Get current refresh token."""
        auth = self.load_auth()
        return auth.get("refresh_token") if auth else None

    def get_user(self) -> Optional[dict]:
        """Get current user info."""
        auth = self.load_auth()
        return auth.get("user") if auth else None


# Global settings instance
_settings: Optional[Settings] = None


def get_settings() -> Settings:
    """Get global settings instance."""
    global _settings
    if _settings is None:
        _settings = Settings()
    return _settings
